'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _request = require('request');

var _request2 = _interopRequireDefault(_request);

var _npm = require('../util/npm');

var _process = require('../util/process');

var _key = require('../util/key');

var _key2 = _interopRequireDefault(_key);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const Service = (name, url, installDirectory, storePath, runnerPort, servicePort, alreadyInstalled = false) => {
  let running = false;
  let starting = false;
  let installed = alreadyInstalled;
  let installing = false;
  let key;
  let whenRunning;
  let process;

  const log = message => console.log(`[${name}] ${message}`);

  const getName = () => name;

  const getState = () => {
    if (running) {
      return 'running';
    }
    if (starting) {
      return 'starting';
    }
    if (installed) {
      return 'installed';
    }
    if (installing) {
      return 'installing';
    }
    return 'registered';
  };

  const validateKey = testKey => testKey === key;

  const install = async () => {
    if (!installed && !installing) {
      log('Installing...');
      installing = true;
      await (0, _npm.install)(url, installDirectory);
      installing = false;
      installed = true;
      log('Installed.');
    }
  };

  const start = async () => {
    if (!running && !starting) {
      key = await (0, _key2.default)();
      starting = true;
      await install();
      log('Starting...');
      const host = `http://localhost:${runnerPort}`;
      process = (0, _process.start)(`node ${name} --host=${host} --port=${servicePort} --key=${key} --store=${storePath}`, _path2.default.join(installDirectory, 'node_modules'), log, log);
      starting = false;
      await new Promise(resolve => {
        whenRunning = resolve;
      });
      running = true;
      log('Running.');
    }
  };

  const stop = async () => {
    if (running && process) {
      await process.stop();
      process = null;
      running = false;
    }
  };

  const restart = async () => {
    if (running) {
      await stop();
      await start();
    }
  };

  const update = async () => {
    const state = getState();
    if (state === 'running' || state === 'registered') {
      await stop();
    }
    installed = false;
    await start();
  };

  const remove = async () => {
    const state = getState();
    if (state === 'running' || state === 'registered') {
      await stop();
      await (0, _npm.uninstall)(name, installDirectory);
      installed = false;
      return true;
    }
    return false;
  };

  const notifyRunning = () => {
    if (whenRunning) {
      whenRunning();
      whenRunning = null;
    }
  };

  const sendMessage = (message, payload = {}) => new Promise((resolve, reject) => {
    if (running) {
      const requestUrl = `http://localhost:${servicePort}/${message}/${key}`;
      _request2.default.post(requestUrl, { json: payload }, (error, response, body) => {
        if (error) {
          reject(error);
        } else if (body) {
          if (body.success) {
            resolve(body.response);
          } else {
            reject(body.error);
          }
        } else {
          reject('Invalid response');
        }
      });
    } else {
      reject('Service is not running.');
    }
  });

  return {
    getName,
    getState,
    validateKey,
    start,
    stop,
    restart,
    update,
    remove,
    notifyRunning,
    sendMessage
  };
};

exports.default = Service;