'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _util = require('util');

var _fsExtra = require('fs-extra');

var _npm = require('../util/npm');

var _Service = require('./Service');

var _Service2 = _interopRequireDefault(_Service);

var _InternalServer = require('./InternalServer');

var _InternalServer2 = _interopRequireDefault(_InternalServer);

var _MetaService = require('./MetaService');

var _MetaService2 = _interopRequireDefault(_MetaService);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const ensureDirPromise = (0, _util.promisify)(_fsExtra.ensureDir);

const ServiceManager = (installDirectory, storePath, firstPort) => {
  const self = {};
  const services = [];
  let initialised = false;
  let internalServer;
  let runnerPort;
  let nextPort = firstPort;

  const getNextPort = () => {
    const port = nextPort;
    nextPort += 1;
    return port;
  };

  const getService = name => services.find(service => service.getName() === name);

  const validateService = key => Boolean(services.find(service => service.validateKey(key)));

  const onServiceRunning = key => {
    const runningService = services.find(service => service.validateKey(key));
    if (runningService) {
      runningService.notifyRunning();
    }
  };

  const initialise = async () => {
    if (!initialised) {
      initialised = true;

      services.push((0, _MetaService2.default)(self, onServiceRunning));

      runnerPort = getNextPort();
      internalServer = (0, _InternalServer2.default)(self, validateService, runnerPort);
      await internalServer.start();

      const servicesPackagePath = _path2.default.join(installDirectory, 'package.json');
      if (_fs2.default.existsSync(servicesPackagePath)) {
        const packageContents = JSON.parse(_fs2.default.readFileSync(servicesPackagePath, 'utf8'));
        if (packageContents.dependencies) {
          Object.keys(packageContents.dependencies).map(name => (0, _Service2.default)(name, packageContents.dependencies[name], installDirectory, _path2.default.join(storePath, name), runnerPort, getNextPort(), true)).forEach(service => services.push(service));
        }

        await Promise.all(services.map(service => service.start()));
      } else {
        await ensureDirPromise(installDirectory);
        await (0, _npm.init)(installDirectory);
      }
    }
  };

  const getStatus = async () => {
    if (initialised) {
      return services.map(service => ({
        name: service.getName(),
        state: service.getState()
      }));
    }
    return Promise.reject('Service manager not initialised.');
  };

  const register = async (name, url) => {
    if (initialised) {
      const existingService = getService(name);
      if (!existingService) {
        const service = (0, _Service2.default)(name, url, installDirectory, runnerPort, getNextPort());
        services.push(service);
        await service.start();
      }
      return Promise.resolve();
    }
    return Promise.reject('Service manager not initialised.');
  };

  const remove = async name => {
    if (initialised) {
      const serviceToRemove = getService(name);
      if (serviceToRemove) {
        const removed = await serviceToRemove.remove();
        if (removed) {
          const serviceIndex = services.findIndex(service => service.getName() === name);
          services.splice(serviceIndex, 1);
        }
      }
      return Promise.resolve();
    }
    return Promise.reject('Service manager not initialised.');
  };

  const sendMessage = async (name, message, payload = {}) => {
    if (initialised) {
      const serviceToMessage = getService(name);
      if (serviceToMessage) {
        return serviceToMessage.sendMessage(message, payload);
      }
      return Promise.reject(`Service ${name} is not registered.`);
    }
    return Promise.reject('Service manager not initialised.');
  };

  const start = async name => {
    const service = getService(name);
    if (service) {
      await service.start();
    }
  };

  const stop = async name => {
    const service = getService(name);
    if (service) {
      await service.stop();
    }
  };

  const restart = async name => {
    const service = getService(name);
    if (service) {
      await service.restart();
    }
  };

  const update = async name => {
    const service = getService(name);
    if (service) {
      await service.update();
    }
  };

  Object.assign(self, {
    getStatus,
    initialise,
    register,
    remove,
    sendMessage,
    start,
    stop,
    restart,
    update
  });

  return self;
};

exports.default = ServiceManager;